/*=========================================================================
*
*  Copyright Insight Software Consortium
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*         http://www.apache.org/licenses/LICENSE-2.0.txt
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*=========================================================================*/
#ifndef sitkCropImageFilter_h
#define sitkCropImageFilter_h

/*
 * WARNING: DO NOT EDIT THIS FILE!
 * THIS FILE IS AUTOMATICALLY GENERATED BY THE SIMPLEITK BUILD PROCESS.
 * Please look at sitkImageFilterTemplate.h.in to make changes.
 */

#include <memory>

#include "sitkBasicFilters.h"
#include "sitkImageFilter.h"

namespace itk {
  namespace simple {

    /**\class CropImageFilter
\brief Decrease the image size by cropping the image by an itk::Size at both the upper and lower bounds of the largest possible region.

CropImageFilter changes the image boundary of an image by removing pixels outside the target region. The target region is not specified in advance, but calculated in BeforeThreadedGenerateData() .

This filter uses ExtractImageFilter to perform the cropping.

\par Wiki Examples:

\li All Examples 

\li Crop an image by specifying the region to throw away
\sa itk::simple::Crop for the procedural interface
\sa itk::CropImageFilter for the Doxygen on the original ITK class.
     */
    class SITKBasicFilters_EXPORT CropImageFilter : public ImageFilter<1> {
    public:
      typedef CropImageFilter Self;

      /** Default Constructor that takes no arguments and initializes
       * default parameters */
      CropImageFilter();

      /** Destructor */
      ~CropImageFilter();

      /** Define the pixels types supported by this filter */
      typedef NonLabelPixelIDTypeList  PixelIDTypeList;



      /**
       * Set/Get the cropping sizes for the upper and lower boundaries.
       */
      SITK_RETURN_SELF_TYPE_HEADER SetLowerBoundaryCropSize ( const std::vector<unsigned int> & LowerBoundaryCropSize ) { this->m_LowerBoundaryCropSize = LowerBoundaryCropSize; return *this; }

      /**
       * Set/Get the cropping sizes for the upper and lower boundaries.
       */
        std::vector<unsigned int> GetLowerBoundaryCropSize() const { return this->m_LowerBoundaryCropSize; }

      /**
       * Set/Get the cropping sizes for the upper and lower boundaries.
       */
      SITK_RETURN_SELF_TYPE_HEADER SetUpperBoundaryCropSize ( const std::vector<unsigned int> & UpperBoundaryCropSize ) { this->m_UpperBoundaryCropSize = UpperBoundaryCropSize; return *this; }

      /**
       * Set/Get the cropping sizes for the upper and lower boundaries.
       */
        std::vector<unsigned int> GetUpperBoundaryCropSize() const { return this->m_UpperBoundaryCropSize; }
      /** Name of this class */
      std::string GetName() const { return std::string ("CropImageFilter"); }

      /** Print ourselves out */
      std::string ToString() const;


      /** Execute the filter on the input image */
      Image Execute ( const Image& image1 );


      /** Execute the filter on the input image with the given parameters */
      Image Execute ( const Image& image1, const std::vector<unsigned int> & lowerBoundaryCropSize, const std::vector<unsigned int> & upperBoundaryCropSize );


    private:

      /** Setup for member function dispatching */

      typedef Image (Self::*MemberFunctionType)( const Image& image1 );
      template <class TImageType> Image ExecuteInternal ( const Image& image1 );



      friend struct detail::MemberFunctionAddressor<MemberFunctionType>;

      nsstd::auto_ptr<detail::MemberFunctionFactory<MemberFunctionType> > m_MemberFactory;


      /* 	odo what does this itk_type attribute do? */
      std::vector<unsigned int>  m_LowerBoundaryCropSize;
      /* 	odo what does this itk_type attribute do? */
      std::vector<unsigned int>  m_UpperBoundaryCropSize;
    };



    /**
     * \brief Decrease the image size by cropping the image by an itk::Size at both the upper and lower bounds of the largest possible region.
     *
     * This function directly calls the execute method of CropImageFilter
     * in order to support a procedural API
     *
     * \sa itk::simple::CropImageFilter for the object oriented interface
     */
     SITKBasicFilters_EXPORT Image Crop ( const Image& image1, const std::vector<unsigned int> & lowerBoundaryCropSize = std::vector<unsigned int>(3, 0), const std::vector<unsigned int> & upperBoundaryCropSize = std::vector<unsigned int>(3, 0) );

  }
}
#endif
