/*=========================================================================
*
*  Copyright Insight Software Consortium
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*         http://www.apache.org/licenses/LICENSE-2.0.txt
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*=========================================================================*/
#ifndef sitkExpNegativeImageFilter_h
#define sitkExpNegativeImageFilter_h

/*
 * WARNING: DO NOT EDIT THIS FILE!
 * THIS FILE IS AUTOMATICALLY GENERATED BY THE SIMPLEITK BUILD PROCESS.
 * Please look at sitkImageFilterTemplate.h.in to make changes.
 */

#include <memory>

#include "sitkBasicFilters.h"
#include "sitkImageFilter.h"

namespace itk {
  namespace simple {

    /**\class ExpNegativeImageFilter
\brief Computes the function exp(-K.x) for each input pixel.

Every output pixel is equal to std::exp(-K.x ). where x is the intensity of the homologous input pixel, and K is a user-provided constant.
\sa itk::simple::ExpNegative for the procedural interface
\sa itk::ExpNegativeImageFilter for the Doxygen on the original ITK class.
     */
    class SITKBasicFilters_EXPORT ExpNegativeImageFilter : public ImageFilter<1> {
    public:
      typedef ExpNegativeImageFilter Self;

      /** Default Constructor that takes no arguments and initializes
       * default parameters */
      ExpNegativeImageFilter();

      /** Destructor */
      ~ExpNegativeImageFilter();

      /** Define the pixels types supported by this filter */
      typedef BasicPixelIDTypeList  PixelIDTypeList;


      /** Name of this class */
      std::string GetName() const { return std::string ("ExpNegativeImageFilter"); }

      /** Print ourselves out */
      std::string ToString() const;


      /** Execute the filter on the input image */
      Image Execute ( const Image& image1 );



    private:

      /** Setup for member function dispatching */

      typedef Image (Self::*MemberFunctionType)( const Image& image1 );
      template <class TImageType> Image ExecuteInternal ( const Image& image1 );

      /** Dispatched methods which calls ExecuteInteral on each component */
      template <class TImageType> Image ExecuteInternalVectorImage ( const Image& image );

      friend struct detail::MemberFunctionAddressor<MemberFunctionType>;
      friend struct detail::ExecuteInternalVectorImageAddressor<MemberFunctionType>;
      nsstd::auto_ptr<detail::MemberFunctionFactory<MemberFunctionType> > m_MemberFactory;


    };



    /**
     * \brief Computes the function exp(-K.x) for each input pixel.
     *
     * This function directly calls the execute method of ExpNegativeImageFilter
     * in order to support a procedural API
     *
     * \sa itk::simple::ExpNegativeImageFilter for the object oriented interface
     */
     SITKBasicFilters_EXPORT Image ExpNegative ( const Image& image1 );

  }
}
#endif
