/*=========================================================================
*
*  Copyright Insight Software Consortium
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*         http://www.apache.org/licenses/LICENSE-2.0.txt
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*=========================================================================*/
#ifndef sitkGaborImageSource_h
#define sitkGaborImageSource_h

/*
 * WARNING: DO NOT EDIT THIS FILE!
 * THIS FILE IS AUTOMATICALLY GENERATED BY THE SIMPLEITK BUILD PROCESS.
 * Please look at sitkImageSourceTemplate.h.in to make changes.
 */

#include <memory>

#include "sitkBasicFilters.h"
#include "sitkImageFilter.h"

namespace itk {
  namespace simple {

    /**\class GaborImageSource
\brief Generate an n-dimensional image of a Gabor filter.

GaborImageSource generates an image of either the real (i.e. symmetric) or complex (i.e. antisymmetric) part of the Gabor filter with the orientation directed along the x-axis. The GaborKernelFunction is used to evaluate the contribution along the x-axis whereas a non-normalized 1-D Gaussian envelope provides the contribution in each of the remaining N dimensions. Orientation can be manipulated via the Transform classes of the toolkit.

The output image may be of any dimension.

This implementation was contributed as a paper to the Insight Journal https://hdl.handle.net/1926/500
\sa itk::simple::GaborImageSource for the procedural interface
\sa itk::GaborImageSource for the Doxygen on the original ITK class.
     */
    class SITKBasicFilters_EXPORT GaborImageSource : public ImageFilter<0> {
    public:
      typedef GaborImageSource Self;

      /** Default Constructor that takes no arguments and initializes
       * default parameters */
      GaborImageSource();

      /** Destructor */
      ~GaborImageSource();

      /** Define the pixels types supported by this filter */
      typedef BasicPixelIDTypeList  PixelIDTypeList;





      /**
       */
      SITK_RETURN_SELF_TYPE_HEADER SetOutputPixelType ( PixelIDValueEnum OutputPixelType ) { this->m_OutputPixelType = OutputPixelType; return *this; }

      /**
       */
        PixelIDValueEnum GetOutputPixelType() const { return this->m_OutputPixelType; }

      /**
       */
      SITK_RETURN_SELF_TYPE_HEADER SetSize ( const std::vector<unsigned int> & Size ) { this->m_Size = Size; return *this; }

      /**
       */
        std::vector<unsigned int> GetSize() const { return this->m_Size; }

      /**
       * Set/Get the the standard deviation in each direction.
       */
      SITK_RETURN_SELF_TYPE_HEADER SetSigma ( const std::vector<double> & Sigma ) { this->m_Sigma = Sigma; return *this; }

      /** Set the values of the Sigma vector all to value */
      SITK_RETURN_SELF_TYPE_HEADER SetSigma( double value ) { this->m_Sigma = std::vector<double>(3, value); return *this; }


      /**
       * Set/Get the the standard deviation in each direction.
       */
        std::vector<double> GetSigma() const { return this->m_Sigma; }

      /**
       * Set/Get the mean in each direction.
       */
      SITK_RETURN_SELF_TYPE_HEADER SetMean ( const std::vector<double> & Mean ) { this->m_Mean = Mean; return *this; }

      /** Set the values of the Mean vector all to value */
      SITK_RETURN_SELF_TYPE_HEADER SetMean( double value ) { this->m_Mean = std::vector<double>(3, value); return *this; }


      /**
       * Set/Get the mean in each direction.
       */
        std::vector<double> GetMean() const { return this->m_Mean; }

      /**
       * Set/Get the modulation frequency of the sine or cosine component.
       */
      SITK_RETURN_SELF_TYPE_HEADER SetFrequency ( double Frequency ) { this->m_Frequency = Frequency; return *this; }

      /**
       * Set/Get the modulation frequency of the sine or cosine component.
       */
        double GetFrequency() const { return this->m_Frequency; }

      /**
       */
      SITK_RETURN_SELF_TYPE_HEADER SetOrigin ( const std::vector<double> & Origin ) { this->m_Origin = Origin; return *this; }

      /**
       */
        std::vector<double> GetOrigin() const { return this->m_Origin; }

      /**
       */
      SITK_RETURN_SELF_TYPE_HEADER SetSpacing ( const std::vector<double> & Spacing ) { this->m_Spacing = Spacing; return *this; }

      /**
       */
        std::vector<double> GetSpacing() const { return this->m_Spacing; }

      /**
       */
      SITK_RETURN_SELF_TYPE_HEADER SetDirection ( std::vector<double> Direction ) { this->m_Direction = Direction; return *this; }

      /**
       */
        std::vector<double> GetDirection() const { return this->m_Direction; }
      /** Name of this class */
      std::string GetName() const { return std::string ("GaborImageSource"); }

      /** Print ourselves out */
      std::string ToString() const;


      /** Execute the filter on the input image */
      Image Execute (  );


      /** Execute the filter on the input image with the given parameters */
      Image Execute ( PixelIDValueEnum outputPixelType, const std::vector<unsigned int> & size, const std::vector<double> & sigma, const std::vector<double> & mean, double frequency, const std::vector<double> & origin, const std::vector<double> & spacing, std::vector<double> direction );


    private:

      /** Setup for member function dispatching */

      typedef Image (Self::*MemberFunctionType)(  );
      template <class TImageType> Image ExecuteInternal (  );



      friend struct detail::MemberFunctionAddressor<MemberFunctionType>;

      nsstd::auto_ptr<detail::MemberFunctionFactory<MemberFunctionType> > m_MemberFactory;


      PixelIDValueEnum  m_OutputPixelType;
      std::vector<unsigned int>  m_Size;
      std::vector<double>  m_Sigma;
      std::vector<double>  m_Mean;
      double  m_Frequency;
      std::vector<double>  m_Origin;
      std::vector<double>  m_Spacing;
      /* Passing a zero sized array, defaults to identiy matrix. The size of the array must exactly match the direction matrix for the dimension of the image. */
      std::vector<double>  m_Direction;



    };



   /**
     * \brief Generate an n-dimensional image of a Gabor filter.
     *
     * This function directly calls the execute method of GaborImageSource
     * in order to support a procedural API
     *
     * \sa itk::simple::GaborImageSource for the object oriented interface
     */
SITKBasicFilters_EXPORT Image GaborSource ( PixelIDValueEnum outputPixelType = itk::simple::sitkFloat32, const std::vector<unsigned int> & size = std::vector<unsigned int>(3, 64), const std::vector<double> & sigma = std::vector<double>(3, 16.0), const std::vector<double> & mean = std::vector<double>(3, 32.0), double frequency = 0.4, const std::vector<double> & origin = std::vector<double>(3, 0.0), const std::vector<double> & spacing = std::vector<double>(3, 1.0), std::vector<double> direction = std::vector<double>() );
  }
}
#endif
