/*=========================================================================
*
*  Copyright Insight Software Consortium
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*         http://www.apache.org/licenses/LICENSE-2.0.txt
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*=========================================================================*/
#ifndef sitkLaplacianRecursiveGaussianImageFilter_h
#define sitkLaplacianRecursiveGaussianImageFilter_h

/*
 * WARNING: DO NOT EDIT THIS FILE!
 * THIS FILE IS AUTOMATICALLY GENERATED BY THE SIMPLEITK BUILD PROCESS.
 * Please look at sitkImageFilterTemplate.h.in to make changes.
 */

#include <memory>

#include "sitkBasicFilters.h"
#include "sitkImageFilter.h"

namespace itk {
  namespace simple {

    /**\class LaplacianRecursiveGaussianImageFilter
\brief Computes the Laplacian of Gaussian (LoG) of an image.

Computes the Laplacian of Gaussian (LoG) of an image by convolution with the second derivative of a Gaussian. This filter is implemented using the recursive gaussian filters.

\par Wiki Examples:

\li All Examples 

\li Compute the Laplacian of Gaussian (LoG) of an image
\sa itk::simple::LaplacianRecursiveGaussian for the procedural interface
\sa itk::LaplacianRecursiveGaussianImageFilter for the Doxygen on the original ITK class.
     */
    class SITKBasicFilters_EXPORT LaplacianRecursiveGaussianImageFilter : public ImageFilter<1> {
    public:
      typedef LaplacianRecursiveGaussianImageFilter Self;

      /** Default Constructor that takes no arguments and initializes
       * default parameters */
      LaplacianRecursiveGaussianImageFilter();

      /** Destructor */
      ~LaplacianRecursiveGaussianImageFilter();

      /** Define the pixels types supported by this filter */
      typedef BasicPixelIDTypeList  PixelIDTypeList;



      /**
       * Set Sigma value. Sigma is measured in the units of image spacing.
       */
      SITK_RETURN_SELF_TYPE_HEADER SetSigma ( double Sigma ) { this->m_Sigma = Sigma; return *this; }

      /**
       * Set Sigma value. Sigma is measured in the units of image spacing.
       */
        double GetSigma() const { return this->m_Sigma; }

      /**
       * Define which normalization factor will be used for the Gaussian \see RecursiveGaussianImageFilter::SetNormalizeAcrossScale
       */
      SITK_RETURN_SELF_TYPE_HEADER SetNormalizeAcrossScale ( bool NormalizeAcrossScale ) { this->m_NormalizeAcrossScale = NormalizeAcrossScale; return *this; }

      /** Set the value of NormalizeAcrossScale to true or false respectfully. */
      SITK_RETURN_SELF_TYPE_HEADER NormalizeAcrossScaleOn() { return this->SetNormalizeAcrossScale(true); }
      SITK_RETURN_SELF_TYPE_HEADER NormalizeAcrossScaleOff() { return this->SetNormalizeAcrossScale(false); }

      /**
       * Define which normalization factor will be used for the Gaussian \see RecursiveGaussianImageFilter::SetNormalizeAcrossScale
       */
        bool GetNormalizeAcrossScale() const { return this->m_NormalizeAcrossScale; }
      /** Name of this class */
      std::string GetName() const { return std::string ("LaplacianRecursiveGaussianImageFilter"); }

      /** Print ourselves out */
      std::string ToString() const;


      /** Execute the filter on the input image */
      Image Execute ( const Image& image1 );


      /** Execute the filter on the input image with the given parameters */
      Image Execute ( const Image& image1, double sigma, bool normalizeAcrossScale );


    private:

      /** Setup for member function dispatching */

      typedef Image (Self::*MemberFunctionType)( const Image& image1 );
      template <class TImageType> Image ExecuteInternal ( const Image& image1 );



      friend struct detail::MemberFunctionAddressor<MemberFunctionType>;

      nsstd::auto_ptr<detail::MemberFunctionFactory<MemberFunctionType> > m_MemberFactory;


      /*  */
      double  m_Sigma;
      /*  */
      bool  m_NormalizeAcrossScale;
    };



    /**
     * \brief Computes the Laplacian of Gaussian (LoG) of an image.
     *
     * This function directly calls the execute method of LaplacianRecursiveGaussianImageFilter
     * in order to support a procedural API
     *
     * \sa itk::simple::LaplacianRecursiveGaussianImageFilter for the object oriented interface
     */
     SITKBasicFilters_EXPORT Image LaplacianRecursiveGaussian ( const Image& image1, double sigma = 1.0, bool normalizeAcrossScale = false );

  }
}
#endif
