/*=========================================================================
*
*  Copyright Insight Software Consortium
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*         http://www.apache.org/licenses/LICENSE-2.0.txt
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*
*=========================================================================*/
#ifndef sitkNotEqualImageFilter_h
#define sitkNotEqualImageFilter_h

/*
 * WARNING: DO NOT EDIT THIS FILE!
 * THIS FILE IS AUTOMATICALLY GENERATED BY THE SIMPLEITK BUILD PROCESS.
 * Please look at sitkBinaryFunctorFilterTemplate.h.in to make changes.
 */

#include <memory>

#include "sitkBasicFilters.h"
#include "sitkImageFilter.h"

namespace itk {
  namespace simple {

    /**\class NotEqualImageFilter
\brief Implements pixel-wise generic operation of two images, or of an image and a constant.

This class is parameterized over the types of the two input images and the type of the output image. It is also parameterized by the operation to be applied. A Functor style is used.

The constant must be of the same type than the pixel type of the corresponding image. It is wrapped in a SimpleDataObjectDecorator so it can be updated through the pipeline. The SetConstant() and GetConstant() methods are provided as shortcuts to set or get the constant value without manipulating the decorator.

\see UnaryFunctorImageFilter TernaryFunctorImageFilter 

\par Wiki Examples:

\li All Examples 

\li Apply a predefined operation to corresponding pixels in two images 

\li Apply a custom operation to corresponding pixels in two images
\sa itk::simple::NotEqual for the procedural interface
\sa itk::BinaryFunctorImageFilter for the Doxygen on the original ITK class.
     */
    class SITKBasicFilters_EXPORT NotEqualImageFilter : public ImageFilter<2> {
    public:
      typedef NotEqualImageFilter Self;

      /** Default Constructor that takes no arguments and initializes
       * default parameters */
      NotEqualImageFilter();

      /** Destructor */
      ~NotEqualImageFilter();

      /** Define the pixels types supported by this filter */
      typedef BasicPixelIDTypeList  PixelIDTypeList;



      /**
       * Set/Get the value used to mark the false pixels of the operator.

       */
      SITK_RETURN_SELF_TYPE_HEADER SetBackgroundValue ( uint8_t BackgroundValue ) { this->m_BackgroundValue = BackgroundValue; return *this; }

      /**
       * Set/Get the value used to mark the false pixels of the operator.

       */
        uint8_t GetBackgroundValue() const { return this->m_BackgroundValue; }

      /**
       * Set/Get the value used to mark the true pixels of the operator.

       */
      SITK_RETURN_SELF_TYPE_HEADER SetForegroundValue ( uint8_t ForegroundValue ) { this->m_ForegroundValue = ForegroundValue; return *this; }

      /**
       * Set/Get the value used to mark the true pixels of the operator.

       */
        uint8_t GetForegroundValue() const { return this->m_ForegroundValue; }
      /** Name of this class */
      std::string GetName() const { return std::string ("NotEqualImageFilter"); }

      /** Print ourselves out */
      std::string ToString() const;


      /** Execute the filter on the input images */
      Image Execute ( const Image& image1, const Image& image2 );


      /** Execute the filter on the input images with the given parameters */
      Image Execute ( const Image& image1, const Image& image2, uint8_t backgroundValue, uint8_t foregroundValue );


      /** Execute the filter with an image and a constant */
      Image Execute ( const Image& image1, double constant );
      Image Execute ( double constant, const Image& image2 );

      /** Execute the filter on an image and a constant with the given parameters */
      Image Execute ( const Image& image1, double constant, uint8_t backgroundValue, uint8_t foregroundValue );
      Image Execute ( double constant, const Image& image2, uint8_t backgroundValue, uint8_t foregroundValue );

    private:

      /** Setup for member function dispatching */

      typedef Image (Self::*MemberFunctionType)( const Image& image1, const Image& image2 );
      template <class TImageType> Image ExecuteInternal ( const Image& image1, const Image& image2 );



      friend struct detail::MemberFunctionAddressor<MemberFunctionType>;

      nsstd::auto_ptr<detail::MemberFunctionFactory<MemberFunctionType> > m_MemberFactory;
      typedef Image (Self::*MemberFunction1Type)( double constant, const Image& image2 );
      template <class TImageType> Image ExecuteInternal ( double constant, const Image& image2 );
      friend struct detail::MemberFunctionAddressor<MemberFunction1Type>;
      nsstd::auto_ptr<detail::MemberFunctionFactory<MemberFunction1Type> > m_MemberFactory1;

      typedef Image (Self::*MemberFunction2Type)( const Image& image1, double constant );
      template <class TImageType> Image ExecuteInternal ( const Image& image1, double constant );
      friend struct detail::MemberFunctionAddressor<MemberFunction2Type>;
      nsstd::auto_ptr<detail::MemberFunctionFactory<MemberFunction2Type> > m_MemberFactory2;


      uint8_t  m_BackgroundValue;
      uint8_t  m_ForegroundValue;
    };


    /**
     * \brief Implements pixel-wise generic operation of two images, or of an image and a constant.
     *
     * This function directly calls the execute method of NotEqualImageFilter
     * in order to support a procedural API
     *
     * \sa itk::simple::NotEqualImageFilter for the object oriented interface
     */
     SITKBasicFilters_EXPORT Image NotEqual ( const Image& image1, const Image& image2, uint8_t backgroundValue = 0u, uint8_t foregroundValue = 1u );

    SITKBasicFilters_EXPORT Image NotEqual ( const Image& image1, double constant, uint8_t backgroundValue = 0u, uint8_t foregroundValue = 1u );
    SITKBasicFilters_EXPORT Image NotEqual ( double constant, const Image& image2, uint8_t backgroundValue = 0u, uint8_t foregroundValue = 1u );
  }
}
#endif
